/********************************************************
  
  RFID-enabled RGB lamp
  
  Author: Jason Webb
  E-mail: webbjk@unk.edu
  Website: http://cs.unk.edu/~webb/
  
  DESCRIPTION --------------------------------------------
  - This program will control an RGB LED according to user
    preferences that are internally stored and associated
    with RFID tags.
  - Originally I intended to have the hardware in place to
    enable a user to pass their tag and control the RGB
    associated with that tag via slide pots. This didn't 
    happen.
  
  NORMAL MODE --------------------------------------------
  - System will monitor pin 15 for serial data from RFID
    reader. When data is received it will read in one tag, 
    verify it, then load the associated color values.
    
  PROGRAMMING MODE ----------------------------------------
  - System only goes into programming mode when module is
    attached (pin 3 goes HIGH).
  - System will monitor pin 15 for serial data from RFID
    reader. When data is received the module is then 
    polled until user hits the "Set" button on the module, 
    at which time the pot values are read and saved and 
    the lamp is adjusted accordingly.
  
  TO-DO --------------------------------------------------
  1) Build physical lamp
  2) Read / save tag and color data to flash mem using
     PROGMEM
  3) Implement Programming Mode
  
**********************************************************/

//-- External libraries ----------------------------------------------------------
#include <SoftwareSerial.h>

//-- Pin assignments --------------------------------------------------------------
//- RFID reader
#include "WProgram.h"
void setup();
void loop();
void normalMode();
void loadPreference( char* tag );
boolean isEqual( char* tag1, char* tag2 );
void changeColor( int color[] );
int rfidEnablePin = 14;  // LOW = enable RFID reader; Analog pin 0
int rfidRxPin     = 15;  // Receives serial data from RFID reader; Analog pin 1
int rfidTxPin     = 16;  // Needed to initialize SoftwareSerial class, not used

//- RGB LED
int redLED   = 9;   // Red channel (output)
int greenLED = 10;  // Green channel (output)
int blueLED  = 11;  // Blue channel (output)

//-- Objects ---------------------------------------------------------------------
SoftwareSerial rfidSerial = SoftwareSerial(rfidRxPin, rfidTxPin);

//-- RFID tag preferences --------------------------------------------------------
int numTags = 2;
int tagLength = 10;
const int maxTags = 20;

char* wrist = {"1F0074FF34"};
char* card = {"360079FD82"};
int colors[maxTags][3] = {{0,100,50}, {200,32,255}};
int currentColor[3] = {255,255,255};

//-- Program constants -----------------------------------------------------------
int red[] = {255,0,0};

//-- Flags -----------------------------------------------------------------------
boolean debug = true;    // TRUE = outputs various data via serial
boolean n_sent = false;  // Indicates whether normal mode header has been sent
boolean p_sent = false;  // Indicates whether prog. mode header has been sent


//---------------------------------------------------------------------//
//-- Set up the pins and variables  -----------------------------------//
//---------------------------------------------------------------------//
void setup() {
    // Ser up serial communication for debugging @ 2400bps
    if(debug) Serial.begin(9600);

  // Set up the RFID reader
  pinMode( rfidRxPin, INPUT );         // Set up pin to receive serial data from RFID reader
  pinMode( rfidEnablePin, OUTPUT );    // Set up the RFID enable pin
  digitalWrite( rfidEnablePin, LOW );  // Activate RFID reader
  rfidSerial.begin( 2400 );            // Start serial communication @ 2400bps with reader
  
    if(debug)    Serial.println("RFID reader initialized");
  
  // Set up the RGB LED
  pinMode( redLED,   OUTPUT );  // Set up the red channel pin
  pinMode( greenLED, OUTPUT );  // Set up the green channel pin
  pinMode( blueLED,  OUTPUT );  // Set up the blue channel pin
  
    if(debug)    Serial.println("RGB lamp initialized");
    
  /* Set up programming mode pins
  pinMode( programPin, INPUT );
  pinMode( setPin,     INPUT );
  pinMode( redPot,     INPUT );
  pinMode( greenPot,   INPUT );
  pinMode( bluePot,    INPUT );
  */
}  


void loop() {
  normalMode();
} 


//---------------------------------------------------------------------//
//-- NORMAL MODE  -----------------------------------------------------//
//---------------------------------------------------------------------//
void normalMode() {
  // Send text header only once
  if(!n_sent && debug) {
    Serial.println();
    Serial.println("NORMAL MODE");
    n_sent = true;
    p_sent = false;
  }
  
  char tag[tagLength]; // Complete RFID tag
  int bytesread, val;  // Number of bytes read from RFID reader  

  // Read in unique non-header bits
  if((val = rfidSerial.read()) == 10) {
    bytesread = 0;
    while(bytesread < tagLength) {
      val = rfidSerial.read();
      if((val == 10)||(val == 13))
        break;
      tag[bytesread] = val;
      bytesread++;
    }
  }

  // Lookup the tag once its been read
  if(bytesread == tagLength) {
    if(debug) {
      Serial.print("Tag found: ");
      for(int i=0; i<tagLength; i++)
        Serial.print(tag[i], BYTE);
      Serial.println();
    }
        
    loadPreference(tag);
  }

}


//---------------------------------------------------------------------//
//-- HELPER FUNCTIONS -------------------------------------------------//  
//---------------------------------------------------------------------//
/**********************************************
 Load preference of an RFID tag
 - Looks up the color value associated with
   the given tag and loads the color
**********************************************/
void loadPreference( char* tag ) {
  if( isEqual(tag,wrist) ) {
    Serial.println("Wrist tag found");
    currentColor[0] = 0;
    currentColor[1] = 200;
    currentColor[2] = 50;
  } else if( isEqual(tag,card) ) {
    Serial.println("Card tag found");
    currentColor[0] = 255;
    currentColor[1] = 50;
    currentColor[2] = 100;
  }
  
  changeColor( currentColor );
}

boolean isEqual( char* tag1, char* tag2 ) {
  boolean equal = true;
        
  for(int i=0; i<tagLength; i++) {
    if( tag1[i] == tag2[i] )  equal = true;
    else  equal = false;
  }
  
  return equal;
}

/********************************************
 Change color of LED
 - Takes in one array of ints, representing
   desired RGB values, then updates LED
*********************************************/
void changeColor( int color[] ) {
  if(debug) {
    Serial.print("Changing color to: {R:");
    Serial.print( color[0] );
    Serial.print("} {G:");
    Serial.print( color[1] );
    Serial.print("} {B:");
    Serial.print( color[2] );
    Serial.println("}");
  }
  
  // Change to new color values
  analogWrite( redLED,   currentColor[0] );  // Update red channel
  analogWrite( greenLED, currentColor[1] );  // Update green channel
  analogWrite( blueLED,  currentColor[2] );  // Update blue channel
}



//---------------------------------------------------------------------//
//-- PROGRAMMING MODE -------------------------------------------------//
//---------------------------------------------------------------------//
/*void programmingMode() {
  if(!p_sent && debug) {
    Serial.println();
    Serial.println("PROGRAMMING MODE");
    p_sent = true;
    n_sent = false;
  }  
  
  char tag[10];               // Complete RFID tag
  int bytesread, val, index;
  
  // Poll RFID reader for header bit
  if((val = rfidSerial.read()) == 10) {
    bytesread = 0;
    while(bytesread < 10) {         // Read in the next 10 bits
      val = rfidSerial.read();
      if((val == 10)||(val == 13))  // Break loop if header or footer bit is read
        break;
      tag[bytesread] = val;
      bytesread++;
    }
  }
  
  // Check if we have read an entire tag
  if(bytesread == 10) {  
      index = validateTag(tag);
      
      if(index >= 0) {
        // Allow LED levels to be adjusted until switch is prsesed
        while( digitalRead(setPin) == LOW ) {
          analogWrite( redLED,   analogRead(redPot)/4 );
          analogWrite( greenLED, analogRead(greenPot)/4 );
          analogWrite( blueLED,  analogRead(bluePot)/4 );
        }      

        // Add in the new color values
        colors[index][0] = analogRead(redPot)/4;
        colors[index][1] = analogRead(greenPot)/4;
        colors[index][2] = analogRead(bluePot)/4;
      }
      
  }
  
}*/



int main(void)
{
	init();

	setup();
    
	for (;;)
		loop();
        
	return 0;
}

